@@ -4,6 +4,19 @@
 #include <fstream>
 #include <sstream>
 #include <Rcpp.h>
+#include "utils.h"
+
+namespace svglite { namespace internal {
+
+template <typename T>
+void write_double(T& stream, double data) {
+  std::streamsize prec = stream.precision();
+  uint8_t newprec = std::fabs(data) >= 1 || data == 0. ? prec : std::ceil(-std::log10(std::fabs(data))) + 1;
+  stream << std::setprecision(newprec) << data << std::setprecision(prec);
+}
+
+}} // namespace svglite::internal
+
 
 class SvgStream {
   public:
@@ -29,6 +42,13 @@ SvgStream& operator<<(SvgStream& object, const T& data) {
   object.write(data);
   return object;
 }
+template <>
+SvgStream& operator<<(SvgStream& object, const double& data) {
+  // Make sure negative zeros are converted to positive zero for
+  // reproducibility of SVGs
+  object.write(dbl_format(data));
+  return object;
+}
 
 class SvgStreamFile : public SvgStream {
   std::ofstream stream_;
@@ -44,7 +64,7 @@ public:
   }
 
   void write(int data)            { stream_ << data; }
-  void write(double data)         { stream_ << data; }
+  void write(double data)         { svglite::internal::write_double(stream_, data); }
   void write(const char* data)    { stream_ << data; }
   void write(char data)           { stream_ << data; }
   void write(const std::string& data) { stream_ << data; }
@@ -80,7 +100,7 @@ public:
   }
 
   void write(int data)                { stream_ << data; }
-  void write(double data)             { stream_ << data; }
+  void write(double data)             { svglite::internal::write_double(stream_, data); }
   void write(const char* data)        { stream_ << data; }
   void write(char data)               { stream_ << data; }
   void write(const std::string& data) { stream_ << data; }
